module CliTools exposing (..)

import Basics.Extra exposing (uncurry)
import MaybeExtra exposing (..)


{-| -}
doWrapString : (String -> Cmd msg) -> (List String -> Maybe String) -> List String -> model -> Maybe ( model, Cmd msg )
doWrapString pt fn args model =
    Maybe.map (\x -> ( model, pt x )) (fn args)


isHelp : List String -> Bool
isHelp l =
    case l of
        [] ->
            True

        fst :: _ ->
            List.member fst [ "help", "-h", "--help" ]


justHelp : String -> List String -> Maybe String
justHelp =
    justIf isHelp


doHelpWith : (String -> Cmd msg) -> String -> List String -> model -> Maybe ( model, Cmd msg )
doHelpWith pt msg args =
    doWrapString pt (justHelp msg) args


isVersion : List String -> Bool
isVersion l =
    case l of
        [] ->
            False

        fst :: _ ->
            List.member fst [ "version", "-v", "--version" ]


justVersionWith : String -> List String -> Maybe String
justVersionWith =
    justIf isVersion


doVersionWith : (String -> Cmd msg) -> String -> List String -> model -> Maybe ( model, Cmd msg )
doVersionWith pt version args =
    doWrapString pt (justVersionWith version) args


mkDefDos : (String -> Cmd msg) -> String -> String -> List (List String -> model -> Maybe ( model, Cmd msg ))
mkDefDos stdout version helpMessage =
    [ doHelpWith stdout helpMessage
    , doVersionWith stdout version
    ]


doElse : (String -> Cmd msg) -> List String -> model -> ( model, Cmd msg )
doElse pt l model =
    case l of
        [] ->
            ( model, pt "Try `help`" )

        fst :: _ ->
            ( model, pt ("Unknown cmd : `" ++ fst ++ "`") )


doOnCmd : String -> (List String -> model -> ( model, Cmd msg )) -> List String -> model -> Maybe ( model, Cmd msg )
doOnCmd cmd doThing args model =
    case args of
        [] ->
            Nothing

        fst :: rest ->
            if cmd == fst then
                Just (doThing rest model)

            else
                Nothing


mkCLi : (String -> Cmd msg) -> String -> String -> List (List String -> model -> Maybe ( model, Cmd msg )) -> List String -> model -> ( model, Cmd msg )
mkCLi stdout version helpMessage dos =
    let
        defDos =
            mkDefDos stdout version helpMessage
    in
    \args -> \model -> justFirstElse (List.map uncurry (defDos ++ dos)) (uncurry (doElse stdout)) ( args, model )
