module Main exposing (main)

import Browser
import Cardano.Data exposing (Data(..), toCbor)
import Cbor.Encode as CborE
import Hex.Convert as Hex
import Html exposing (Html, button, div, h1, text)
import Html.Attributes exposing (class)
import Html.Events exposing (onClick)
import Integer exposing (Integer)
import Natural



-- MAIN


main : Program () Model Msg
main =
    Browser.sandbox { init = init, update = update, view = view }



-- MODEL


type alias Model =
    Int


init : Model
init =
    0


modelToData model =
    case Integer.fromInt model of
        Just i ->
            Int i

        _ ->
            Int Integer.one



-- type Data
--     = Constr Natural (List Data)
--     | Map (List ( Data, Data ))
--     | List (List Data)
--     | Int Integer
--     | Bytes (Bytes Any)


asString : Data -> String
asString d =
    case d of
        Constr n _ ->
            "Constr " ++ Natural.toString n

        Map l ->
            "Map " ++ String.fromInt (List.length l)

        _ ->
            "OTHER "



-- UPDATE


type Msg
    = Increment
    | Decrement


update : Msg -> Model -> Model
update msg model =
    case msg of
        Increment ->
            model * model + 1

        Decrement ->
            model - 1



-- VIEW


view : Model -> Html Msg
view model =
    div [ class "container" ]
        [ div [ class "row" ]
            [ div [ class "col-sm-8" ] [ h1 [] [ text "Title" ] ] ]
        , div [ class "row" ]
            [ div [ class "col-sm-2" ] [ button [ onClick Increment ] [ text "+" ] ]
            , div [ class "col-sm-2" ] [ text (Hex.toString (CborE.encode (toCbor (modelToData model)))) ]
            , div [ class "col-sm-2" ] [ button [ onClick Decrement ] [ text "-" ] ]
            ]
        ]
